import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter_mutation/mutation.dart';
import 'package:flutter_mutation/mutation_cache.dart';
import 'package:flutter_mutation/mutation_types.dart';

class MutationKey<R> {
  Mutation<R>? get _mutation => MutationCache.instance.getMutation(this);

  R? get data => _mutation?.data;

  List<R>? get dataList => _mutation?.dataList;

  Object? get error => _mutation?.error;

  bool? get isLoading => _mutation?.isLoading;

  bool? get isInitialized => _mutation?.isInitilized;

  final bool static;

  MutationKey() : static = true {
    MutationCache.instance.retain(this);
  }

  MutationKey.autoClose() : static = false;

  @override
  String toString() {
    return shortHash(this);
  }

  Future<R>? mutate(FutureOr<R> future, {bool append = false}) {
    return _mutation?.mutate(future, append: append);
  }

  Future<bool>? updateInitialize(MutationGetInitialValueCallback<R> callback) {
    return _mutation?.updateInitialize(callback);
  }

  bool? clear() {
    return _mutation?.clear();
  }

  bool? clearError() {
    return _mutation?.clearError();
  }

  bool? clearData() {
    return _mutation?.clearData();
  }

  void close() {
    final mutation = _mutation;
    if (mutation != null) {
      MutationCache.instance.remove(this);
      mutation.close();
    }
  }

  void addObserve({
    MutationOnUpdateDataCallback<R>? onUpdateData,
    MutationOnUpdateErrorCallback? onUpdateError,
    MutationOnUpdateInitializedCallback? onUpdateInitialized,
    MutationOnUpdateLoadingCallback? onUpdateLoading,
    MutationOnOpenCallback<R>? onOpen,
    MutationOnCloseCallback<R>? onClose,
  }) {
    MutationCache.instance.addObserve(this,
        onUpdateData: onUpdateData,
        onUpdateError: onUpdateError,
        onUpdateInitialized: onUpdateInitialized,
        onUpdateLoading: onUpdateLoading,
        onOpen: onOpen,
        onClose: onClose);
  }

  bool removeObserve({
    MutationOnUpdateDataCallback<R>? onUpdateData,
    MutationOnUpdateErrorCallback? onUpdateError,
    MutationOnUpdateInitializedCallback? onUpdateInitialized,
    MutationOnUpdateLoadingCallback? onUpdateLoading,
    MutationOnOpenCallback<R>? onOpen,
    MutationOnCloseCallback<R>? onClose,
  }) {
    return MutationCache.instance.removeObserve(this,
        onUpdateData: onUpdateData,
        onUpdateError: onUpdateError,
        onUpdateInitialized: onUpdateInitialized,
        onUpdateLoading: onUpdateLoading,
        onOpen: onOpen,
        onClose: onClose);
  }
}
